@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,8 @@
 
 typedef __u32 cap_t;
 
-#define CAP_TO_MASK(x) (1 << (x))
+#undef CAP_TO_MASK
+#define CAP_TO_MASK(x) (1 << ((x) & 31)) /* mask for indexed __u32 */
 #define cap_raise(c, flag) (c |=  CAP_TO_MASK(flag))
 #define cap_lower(c, flag) (c &=  CAP_TO_MASK(flag))
 
@@ -42,9 +43,9 @@ typedef struct {
  */
 int get_cap_mask(char *name, unsigned long *mask);
 
-/** Apply capability mask to VE.
+/** Apply capability mask to CT.
  *
- * @param veid		VE id.
+ * @param veid		CT ID.
  * @param cap		capability mask.
  * @return		0 on success.
  */
