@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -49,7 +49,8 @@ typedef struct {
  * @param dev		devices list.
  * @return		0 on success.
  */
-int vps_set_devperm(vps_handler *h, envid_t veid, char *root, dev_param *dev);
+int vps_set_devperm(vps_handler *h, envid_t veid, const char *root,
+		dev_param *dev);
 int set_devperm(vps_handler *h, envid_t veid, dev_res *dev);
 int add_dev_param(dev_param *dev, dev_res *res);
 void free_dev_param(dev_param *dev);
