@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,9 +38,11 @@
 #include "lock.h"
 #include "vps_configure.h"
 #include "modules.h"
+#include "io.h"
 
 extern struct mod_action g_action;
-extern int do_enter(vps_handler *h, envid_t veid, char *root);
+extern int do_enter(vps_handler *h, envid_t veid, const char *root,
+			int argc, char **argv);
 
 int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 	vps_param *param)
@@ -55,7 +57,7 @@ int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 			break;
 		if (c == '?')
 			return VZ_INVALID_PARAMETER_VALUE;
-		ret = vps_parse_opt(veid, param, c, optarg, &g_action);
+		ret = vps_parse_opt(veid, opt, param, c, optarg, &g_action);
 		if (!ret)
 			continue;
 		else if (ret == ERR_INVAL || ret == ERR_INVAL_SKIP ||
@@ -90,16 +92,17 @@ int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 	return 0;
 }
 
-static int parse_start_opt(int argc, char **argv, vps_param *param)
+static int parse_startstop_opt(int argc, char **argv, vps_param *param,
+		int start, int stop)
 {
 	int c, ret = 0;
 	struct option start_options[] = {
-	{"force", no_argument, NULL, PARAM_FORCE},
-	{"skip_ve_setup", no_argument, NULL, PARAM_SKIP_VE_SETUP},
-	{"wait", no_argument, NULL, PARAM_WAIT},
-
-	{ NULL, 0, NULL, 0 }
-};
+		{"force", no_argument, NULL, PARAM_FORCE},
+		{"skip_ve_setup", no_argument, NULL, PARAM_SKIP_VE_SETUP},
+		{"wait", no_argument, NULL, PARAM_WAIT},
+		{"fast", no_argument, NULL, PARAM_FAST},
+		{ NULL, 0, NULL, 0 }
+	};
 
 	while (1) {
 		c = getopt_long (argc, argv, "", start_options, NULL);
@@ -107,13 +110,28 @@ static int parse_start_opt(int argc, char **argv, vps_param *param)
 			break;
 		switch (c) {
 		case PARAM_FORCE:
-			param->opt.start_force = YES;
+			if (start)
+				param->opt.start_force = YES;
+			else
+				ret = VZ_INVALID_PARAMETER_SYNTAX;
 			break;
 		case PARAM_SKIP_VE_SETUP:
-			param->opt.skip_setup = YES;
+			if (start)
+				param->opt.skip_setup = YES;
+			else
+				ret = VZ_INVALID_PARAMETER_SYNTAX;
 			break;
 		case PARAM_WAIT:
-			param->res.misc.wait = YES;
+			if (start)
+				param->res.misc.wait = YES;
+			else
+				ret = VZ_INVALID_PARAMETER_SYNTAX;
+			break;
+		case PARAM_FAST:
+			if (stop)
+				param->opt.fast_kill = YES;
+			else
+				ret = VZ_INVALID_PARAMETER_SYNTAX;
 			break;
 		default:
 			ret = VZ_INVALID_PARAMETER_SYNTAX;
@@ -144,30 +162,6 @@ static int start(vps_handler *h, envid_t veid, vps_param *g_p,
 	return ret;
 }
 
-static int parse_stop_opt(int argc, char **argv, vps_opt *opt)
-{
-	int c, ret = 0;
-	struct option stop_options[] = {
-	{"fast",	no_argument, NULL, PARAM_FAST},
-	{ NULL, 0, NULL, 0 }
-};
-
-	while (1) {
-		c = getopt_long (argc, argv, "", stop_options, NULL);
-		if (c == -1)
-			break;
-		switch (c) {
-		case PARAM_FAST:
-			opt->fast_kill = YES;
-			break;
-		default		:
-			ret = VZ_INVALID_PARAMETER_SYNTAX;
-			break;
-		}
-	}
-	return ret;
-}
-
 static int stop(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *cmd_p)
 {
 	int ret;
@@ -185,7 +179,17 @@ static int stop(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *cmd_p)
 static int restart(vps_handler *h, envid_t veid, vps_param *g_p,
 	vps_param *cmd_p)
 {
-	return vps_restart(h, veid, g_p);
+	int ret;
+
+	logger(0, 0, "Restarting container");
+
+	if (vps_is_run(h, veid)) {
+		ret = stop(h, veid, g_p, cmd_p);
+		if (ret != 0)
+			return ret;
+	}
+
+	return start(h, veid, g_p, cmd_p);
 }
 
 static int parse_create_opt(envid_t veid, int argc, char **argv,
@@ -382,45 +386,81 @@ err_syntax:
 	logger(-1, 0, "Invalid syntax: only one sub command may be used");
 	return VZ_INVALID_PARAMETER_SYNTAX;
 }
+
+/* Check parameters that can't be set on running CT */
 int check_set_mode(vps_handler *h, envid_t veid, int setmode, int apply,
 	vps_res *new_res, vps_res *old_res)
 {
-	int err = 0;
+	int found = 0;
+	int loud = (setmode != SET_RESTART);
 
-	/* Check parameters that can't be set on running CT */
-	if (new_res->cap.on || new_res->cap.off) {
-		logger(-1, 0, "Unable to set capability on running container");
-		if (setmode == SET_RESTART)
-			goto restart_ve;
-		else if (setmode != SET_IGNORE)
-			err = -1;
+	/* If some caps are set and they differ from the old ones */
+	if ( (new_res->cap.on &&
+				(new_res->cap.on != old_res->cap.on)) ||
+		(new_res->cap.off &&
+				(new_res->cap.off != old_res->cap.off)) ) {
+		if (loud)
+			logger(-1, 0, "Unable to set capability "
+					"on a running container");
+		found++;
 	}
-	if (new_res->env.ipt_mask) {
-		if (!old_res->env.ipt_mask ||
+	/* If iptables mask is set and it differs from the old one.
+	 * FIXME: we don't catch the case when the new set is empty
+	 * and the old one is not (vzctl set --iptables '') */
+	if (new_res->env.ipt_mask &&
 			new_res->env.ipt_mask != old_res->env.ipt_mask)
-		{
+	{
+		if (loud)
 			logger(-1, 0, "Unable to set iptables "
-					"on running container");
-			if (setmode == SET_RESTART)
-				goto restart_ve;
-			else if (setmode != SET_IGNORE)
-				err = -1;
+					"on a running container");
+		found++;
+	}
+	/* If some features are set (to either on or off)
+	 * and the new mask differs from the old one */
+	if (new_res->env.features_known &&
+			new_res->env.features_mask !=
+			old_res->env.features_mask)
+	{
+		if (loud)
+			logger(-1, 0, "Unable to set features "
+					"on a running container");
+		found++;
+	}
+	/* Setting/changing quotaugidlimit */
+	if (new_res->dq.ugidlimit) {
+		if (!old_res->dq.ugidlimit) {
+			if (loud)
+				logger(-1, 0, "Unable to turn on second-level"
+					" disk quota on a running container");
+			found++;
+		} else if (*new_res->dq.ugidlimit != *old_res->dq.ugidlimit) {
+			if (loud)
+				logger(-1, 0, "Unable to change quota ugid "
+					"limit on a running container");
+			found++;
 		}
 	}
-	if (err && setmode == SET_NONE) {
-		logger(-1, 0, "WARNING: Some of the parameters"
-			" could not be applied to a running container.\n"
-			"\tPlease consider using --setmode option");
-	}
-	return err;
 
-restart_ve:
-	return 1;
+	if (!found)
+		return 0;
+
+	switch (setmode) {
+		case SET_RESTART:
+			return 1;
+		case SET_IGNORE:
+			return -1;
+		case SET_NONE:
+		default:
+			logger(-1, 0, "WARNING: Some of the parameters could "
+				"not be applied to a running container.\n"
+				"\tPlease consider using --setmode option");
+			return 0;
+	}
 }
 
 static void merge_apply_param(vps_param *old, vps_param *new, char *cfg)
 {
-#define FREE_STR(x)	if ((x) != NULL) { free(x); x = NULL;}
+#define FREE_STR(x)	free(x); x = NULL;
 
 	FREE_STR(new->res.fs.root_orig)
 	FREE_STR(new->res.fs.private_orig)
@@ -488,6 +528,8 @@ static int set_ve0(vps_handler *h, vps_param *g_p,
 	ret = vps_set_ublimit(h, 0, ub);
 	if (ret)
 		return ret;
+	if ((ret = ve_ioprio_set(h, 0, &cmd_p->res.io)))
+		return ret;
 	if (cpu != NULL)
 		if ((ret = hn_set_cpu(cpu)))
 			return ret;
@@ -521,14 +563,14 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 			return ret;
 	}
 	/* Reset UB parameters from config  */
-	if (cmd_p->opt.reset_ub == YES) {
+	if (cmd_p->opt.reset_ub == YES && h != NULL) {
 		ret = vps_set_ublimit(h, veid, &vps_p->res.ub);
 		cmd_p->opt.save = NO; // suppress savewarning
 		return ret;
 	}
 	if (cmd_p->opt.setmode == SET_RESTART && !cmd_p->opt.save) {
-		logger(-1, 0, "it's impossible to use"
-			" restart mode without --save");
+		logger(-1, 0, "Error: --setmode restart doesn't make sense"
+			" without --save flag");
 		return VZ_INVALID_PARAMETER_SYNTAX;
 	}
 	if (cmd_p->res.name.name != NULL) {
@@ -545,7 +587,7 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 			return ret;
 		}
 	}
-	is_run = vps_is_run(h, veid);
+	is_run = h != NULL && vps_is_run(h, veid);
 	if (is_run) {
 		if (cmd_p->res.fs.private_orig != NULL) {
 			free(cmd_p->res.fs.private_orig);
@@ -570,11 +612,10 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 		dist_name = get_dist_name(&g_p->res.tmpl);
 		if ((ret = read_dist_actions(dist_name, DIST_DIR, actions)))
 			return ret;
-		if (dist_name != NULL)
-			free(dist_name);
+		free(dist_name);
 	}
 	/* Setup password */
-	if (!list_empty(&cmd_p->res.misc.userpw)) {
+	if (h != NULL && !list_empty(&cmd_p->res.misc.userpw)) {
 		if (!is_run)
 			if ((ret = vps_start(h, veid, g_p,
 				SKIP_SETUP|SKIP_ACTION_SCRIPT, NULL)))
@@ -611,7 +652,7 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 				goto err;
 			} else {
 				merge_vps_param(g_p, cmd_p);
-				ret = vps_restart(h, veid, g_p);
+				ret = restart(h, veid, g_p, cmd_p);
 				goto err;
 			}
 		}
@@ -620,7 +661,7 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 		STATE_RUNNING, SKIP_NONE, &g_action);
 err:
 	free_dist_actions(actions);
-	if (actions != NULL) free(actions);
+	free(actions);
 
 	return ret;
 }
@@ -637,11 +678,9 @@ static int umount(vps_handler *h, envid_t veid, vps_param *g_p,
 	return vps_umount(h, veid, g_p->res.fs.root, 0);
 }
 
-static int enter(vps_handler *h, envid_t veid, vps_param *g_p,
-	vps_param *cmd_p)
+static int enter(vps_handler *h, envid_t veid, const char *root,
+		int argc, char **argv)
 {
-	char *root = g_p->res.fs.root;
-
 	set_log_file(NULL);
 	if (check_var(root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
@@ -649,11 +688,15 @@ static int enter(vps_handler *h, envid_t veid, vps_param *g_p,
 		logger(-1, 0, "Container is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
-	return do_enter(h, veid, root);
+	if (argc > 0) {
+		/* omit "--exec" argument */
+		argc -= 1; argv += 1;
+	}
+	return do_enter(h, veid, root, argc, argv);
 }
 
-static int exec(vps_handler *h, int action, envid_t veid, char *root, int argc,
-	char **argv)
+static int exec(vps_handler *h, act_t action, envid_t veid, const char *root,
+		int argc, char **argv)
 {
 	int mode;
 	char **arg = NULL;
@@ -716,7 +759,7 @@ static int restore(vps_handler *h, envid_t veid, vps_param *g_p,
 
 static int show_status(vps_handler *h, envid_t veid, vps_param *param)
 {
-	int exist = 0, mounted = 0, run = 0;
+	int exist = 0, mounted = 0, run = 0, suspended = 0;
 	char buf[STR_SIZE];
 	fs_param *fs = &param->res.fs;
 
@@ -725,10 +768,16 @@ static int show_status(vps_handler *h, envid_t veid, vps_param *param)
 		exist = 1;
 	mounted = vps_is_mounted(fs->root);
 	run = vps_is_run(h, veid);
-	printf("CTID %d %s %s %s\n", veid,
+	if (exist == 1) {
+		get_dump_file(veid, param->res.cpt.dumpdir, buf, sizeof(buf));
+		if (stat_file(buf))
+			suspended = 1;
+	}
+	printf("CTID %d %s %s %s%s\n", veid,
 		exist ? "exist" : "deleted",
 		mounted ? "mounted" : "unmounted",
-		run ? "running" : "down");
+		run ? "running" : "down",
+		suspended ? " suspended" : "");
 	return 0;
 }
 
@@ -747,7 +796,7 @@ static int parse_custom_opt(envid_t veid, int argc, char **argv,
 	return ret;
 }
 
-int parse_action_opt(envid_t veid, int action, int argc, char *argv[],
+int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 	vps_param *param, const char *name)
 {
 	int ret = 0;
@@ -759,11 +808,30 @@ int parse_action_opt(envid_t veid, int action, int argc, char *argv[],
 	case ACTION_CREATE:
 		ret = parse_create_opt(veid, argc, argv, param);
 		break;
-	case ACTION_STOP:
-		ret = parse_stop_opt(argc, argv, &param->opt);
-		break;
 	case ACTION_START:
-		ret = parse_start_opt(argc, argv, param);
+		ret = parse_startstop_opt(argc, argv, param, 1, 0);
+		break;
+	case ACTION_STOP:
+		ret = parse_startstop_opt(argc, argv, param, 0, 1);
+		break;
+	case ACTION_RESTART:
+		ret = parse_startstop_opt(argc, argv, param, 1, 1);
+		break;
+	case ACTION_ENTER:
+		if (argc >= 2) {
+			if (!strcmp(argv[1], "--exec")) {
+				if ((argc == 2) || (*argv[2] == '\0')) {
+					fprintf(stderr,
+						"No command line "
+						"given for --exec\n");
+					ret = VZ_INVALID_PARAMETER_SYNTAX;
+				}
+			} else {
+				fprintf(stderr,
+					"Invalid option '%s'\n", argv[1]);
+				ret = VZ_INVALID_PARAMETER_SYNTAX;
+			}
+		}
 		break;
 	case ACTION_DESTROY:
 		break;
@@ -779,7 +847,7 @@ int parse_action_opt(envid_t veid, int action, int argc, char *argv[],
 		break;
 	case ACTION_RUNSCRIPT:
 		if (argc < 2) {
-			fprintf(stderr, "Script does not specified\n");
+			fprintf(stderr, "Script not specified\n");
 			ret = VZ_INVALID_PARAMETER_SYNTAX;
 		}
 		break;
@@ -804,7 +872,7 @@ int parse_action_opt(envid_t veid, int action, int argc, char *argv[],
 	return ret;
 }
 
-int run_action(envid_t veid, int action, vps_param *g_p, vps_param *vps_p,
+int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 	vps_param *cmd_p, int argc, char **argv, int skiplock)
 {
 	vps_handler *h = NULL;
@@ -813,8 +881,16 @@ int run_action(envid_t veid, int action, vps_param *g_p, vps_param *vps_p,
 	char fname[STR_SIZE];
 
 	ret = 0;
-	if ((h = vz_open(veid)) == NULL)
-		return VZ_BAD_KERNEL;
+	if ((h = vz_open(veid)) == NULL) {
+		/* Accept to run "set --save --force" on non-openvz
+		 * kernel */
+		if (action != ACTION_SET ||
+		    cmd_p->opt.save_force != YES ||
+		    cmd_p->opt.save != YES)
+		{
+			return VZ_BAD_KERNEL;
+		}
+	}
 	if (action != ACTION_EXEC &&
 		action != ACTION_EXEC2 &&
 		action != ACTION_EXEC3 &&
@@ -867,12 +943,19 @@ int run_action(envid_t veid, int action, vps_param *g_p, vps_param *vps_p,
 			ret = set(h, veid, g_p, vps_p, cmd_p);
 		if (cmd_p->opt.save == YES) {
 			get_vps_conf_path(veid, fname, sizeof(fname));
-			vps_save_config(veid, fname, cmd_p, vps_p, &g_action);
-			logger(0, 0, "Saved parameters for CT %d", veid);
+			/* Warn if config does not exist */
+			if (!stat_file(fname))
+				logger(-1, errno, "WARNING: %s not found",
+					fname);
+			ret = vps_save_config(veid, fname,
+					cmd_p, vps_p, &g_action);
+			if (ret == 0)
+				logger(0, 0, "Saved parameters "
+						"for CT %d", veid);
 		} else if (cmd_p->opt.save != NO) {
 			if (list_empty(&cmd_p->res.misc.userpw)) {
 				logger(0, 0, "WARNING: Settings were not saved"
-				" and will be resetted to original values on"
+				" and will be reset to original values at the"
 				" next start (use --save flag)");
 			}
 		}
@@ -881,7 +964,7 @@ int run_action(envid_t veid, int action, vps_param *g_p, vps_param *vps_p,
 		ret = show_status(h, veid, g_p);
 		break;
 	case ACTION_ENTER:
-		ret = enter(h, veid, g_p, cmd_p);
+		ret = enter(h, veid, g_p->res.fs.root, argc, argv);
 		break;
 	case ACTION_EXEC:
 	case ACTION_EXEC2:
