@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,9 +38,9 @@ int _proc_title_len;
 
 void init_modules(struct mod_action *action, const char *name);
 void free_modules(struct mod_action *action);
-int parse_action_opt(envid_t veid, int action, int argc, char *argv[],
+int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 	vps_param *param, const char *name);
-int run_action(envid_t veid, int action, vps_param *g_p, vps_param *vps_p,
+int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 	vps_param *cmd_p, int argc, char **argv, int skiplock);
 
 void version(FILE *fp)
@@ -54,20 +54,22 @@ void usage(int rc)
 	FILE *fp = rc ? stderr : stdout;
 
 	version(fp);
-	fprintf(fp, "Copyright (C) 2000-2008, Parallels, Inc.\n");
+	fprintf(fp, "Copyright (C) 2000-2010, Parallels, Inc.\n");
 	fprintf(fp, "This program may be distributed under the terms of the GNU GPL License.\n\n");
-	fprintf(fp, "Usage: vzctl [options] <command> <veid> [parameters]\n"
-"vzctl destroy | mount | umount | stop | restart | status | enter <veid>\n"
-"vzctl create <veid> [--ostemplate <name>] [--config <name>]\n"
+	fprintf(fp, "Usage: vzctl [options] <command> <ctid> [parameters]\n"
+"vzctl destroy | mount | umount | stop | restart | status\n"
+"vzctl enter <ctid> [--exec <command> [arg ...]]\n"
+"vzctl create <ctid> [--ostemplate <name>] [--config <name>]\n"
 "   [--private <path>] [--root <path>] [--ipadd <addr>] | [--hostname <name>]\n"
-"vzctl start <veid> [--force] [--wait]\n"
-"vzctl exec | exec2 <veid> <command> [arg ...]\n"
-"vzctl runscript <veid> <script>\n"
-"vzctl chkpnt <veid> [--dumpfile <name>]\n"
-"vzctl restore <veid> [--dumpfile <name>]\n"
-"vzctl set <veid> [--save] [--setmode restart|ignore]\n"
+"vzctl start <ctid> [--force] [--wait]\n"
+"vzctl exec | exec2 <ctid> <command> [arg ...]\n"
+"vzctl runscript <ctid> <script>\n"
+"vzctl chkpnt <ctid> [--dumpfile <name>]\n"
+"vzctl restore <ctid> [--dumpfile <name>]\n"
+"vzctl set <ctid> [--save] [--force] [--setmode restart|ignore]\n"
 "   [--ipadd <addr>] [--ipdel <addr>|all] [--hostname <name>]\n"
-"   [--nameserver <addr>] [--searchdomain <name>] [--onboot yes|no]\n"
+"   [--nameserver <addr>] [--searchdomain <name>]\n"
+"   [--onboot yes|no] [--bootorder <N>]\n"
 "   [--userpasswd <user>:<passwd>] [--cpuunits <N>] [--cpulimit <N>] [--cpus <N>]\n"
 "   [--diskspace <soft>[:<hard>]] [--diskinodes <soft>[:<hard>]]\n"
 "   [--quotatime <N>] [--quotaugidlimit <N>]\n"
@@ -91,7 +93,9 @@ void usage(int rc)
 "   --oomguarpages P[:P]	--lockedpages P[:P]	--privvmpages P[:P]\n"
 "   --shmpages P[:P]	--numfile N[:N]		--numflock N[:N]\n"
 "   --numpty N[:N]	--numsiginfo N[:N]	--dcachesize N[:N]\n"
-"   --numiptent N[:N]	--physpages P[:P]	--avnumproc N[:N]\n");
+"   --numiptent N[:N]	--physpages P[:P]	--avnumproc N[:N]\n"
+"   --swappages P[:P]\n"
+);
 	memset(&mod, 0, sizeof(mod));
 	set_log_level(0);
 	init_modules(&mod, NULL);
@@ -102,7 +106,7 @@ void usage(int rc)
 
 int main(int argc, char *argv[], char *envp[])
 {
-	int action = 0;
+	act_t action = -1;
 	int verbose = 0;
 	int verbose_tmp;
 	int verbose_custom = 0;
@@ -165,31 +169,23 @@ int main(int argc, char *argv[], char *envp[])
 	if (!strcmp(argv[1], "set")) {
 		init_modules(&g_action, "set");
 		action = ACTION_SET;
-	//	status = ST_SET;
 	} else if (!strcmp(argv[1], "create")) {
 		init_modules(&g_action, "create");
 		action = ACTION_CREATE;
-	//	status = ST_CREATE;
 	} else if (!strcmp(argv[1], "start")) {
 		init_modules(&g_action, "set");
 		action = ACTION_START;
-	//	status = ST_START;
 	} else if (!strcmp(argv[1], "stop")) {
 		init_modules(&g_action, "set");
 		action = ACTION_STOP;
-	//	status = ST_STOP;
 	} else if (!strcmp(argv[1], "restart")) {
 		action = ACTION_RESTART;
-	//	status = ST_RESTART;
-	} else if (!strcmp(argv[1], "destroy")) {
+	} else if (!strcmp(argv[1], "destroy") || !strcmp(argv[1], "delete")) {
 		action = ACTION_DESTROY;
-	//	status = ST_DESTROY;
 	} else if (!strcmp(argv[1], "mount")) {
 		action = ACTION_MOUNT;
-	//	status = ST_MOUNT;
 	} else if (!strcmp(argv[1], "umount")) {
 		action = ACTION_UMOUNT;
-	//	status = ST_UMOUNT;
 	} else if (!strcmp(argv[1], "exec3")) {
 		action = ACTION_EXEC3;
 	} else if (!strcmp(argv[1], "exec2")) {
@@ -309,7 +305,7 @@ error:
 	free_vps_param(vps_p);
 	free_vps_param(cmd_p);
 	free_log();
-	if (name != NULL) free(name);
+	free(name);
 
 	return ret;
 }
